Package: BsplineQuantReg
Type: Package
Title: 'Constrained Quantile Regression with B-Splines'
Version: 0.2.0
Date: 2026-07-18
Authors@R: person("Alexandre", "Abbes", email = "alexandre.abbes@proton.me", role = c("aut", "cre"))
Description: Quantile regression with B-splines under shape constraints.
    The initial version with cubic splines is now augmented with splines
    of degree 1 to 4. Constraints for degrees 3 (monotone) and 4
    (monotone and convex) use the Karlin-Studden SOCP characterization
    for the sign of the polynomial, while other constraints applied
    at the knots are added as linear problems. The method for cubic
    splines is described in Abbes (2026) <doi:10.5281/zenodo.17427913>.
    Other formulations are simple consequences of the other given
    references. This R implementation is intended for demonstration
    and prototyping. All B-spline and polynomial functions have been
    rewritten for consistency. An equivalent Python package is available
    at <https://pypi.org/project/BsplineQuantRegpy/>.
License: GPL-3
URL: https://github.com/alexandreabbes/BsplineQuantReg,
        https://doi.org/10.5281/zenodo.17427913
BugReports: https://github.com/alexandreabbes/BsplineQuantReg/issues
Depends: R (>= 3.5.0)
Imports: CVXR
Suggests: cobs, quantreg
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-21 16:16:53 UTC; abbes
Author: Alexandre Abbes [aut, cre]
Maintainer: Alexandre Abbes <alexandre.abbes@proton.me>
Repository: CRAN
Date/Publication: 2026-07-21 17:30:02 UTC
Built: R 4.4.3; ; 2026-07-21 19:55:29 UTC; unix
