# Generated by roxygen2: do not edit by hand

S3method(print,callable_spline)
S3method(print,quantile_spline)
export(Bspline_base)
export(Bspline_deriv)
export(SplineConstQuantRegBs1)
export(SplineConstQuantRegBs2)
export(SplineConstQuantRegBs3)
export(SplineConstQuantRegBs4)
export(SplineCubicQuant)
export(SplineLinearQuant)
export(SplineQuadraticQuant)
export(SplineQuarticQuant)
export(Spline_der_knot)
export(bs_direct)
export(bspline_to_deriv_coeffs_cubic)
export(bspline_to_deriv_coeffs_lin)
export(bspline_to_deriv_coeffs_quad)
export(bspline_to_deriv_coeffs_quart)
export(change_polynomial_base_taylor)
export(make_spline)
export(poly_eval)
export(polyadd)
export(polyderiv)
export(polymul)
export(quantile_spline)
export(reduce_pol)
export(spline_eval)
export(test_karlin_simple)
export(view_basis)
import(CVXR)
importFrom(graphics,abline)
importFrom(graphics,grid)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(stats,knots)
importFrom(stats,quantile)
importFrom(stats,rnorm)
